@@ -15,7 +15,11 @@ def read_default(name=None):
     if not _defaults:
         if os.path.exists('/usr/share/python/debian_defaults'):
             config = SafeConfigParser()
-            config.readfp(file('/usr/share/python/debian_defaults'))
+            try:
+                config.readfp(file('/usr/share/python/debian_defaults'))
+            except IOError, msg:
+                print msg
+                sys.exit(1)
             _defaults = config
     if _defaults and name:
         try:
@@ -25,7 +29,7 @@ def read_default(name=None):
         return value
     return None
 
-def parse_versions(vstring):
+def parse_versions(vstring, add_exact=False):
     import operator
     operators = { None: operator.eq, '=': operator.eq,
                   '>=': operator.ge, '<=': operator.le,
@@ -33,7 +37,8 @@ def parse_versions(vstring):
                   }
     vinfo = {}
     exact_versions = set([])
-    version_range = set(supported_versions(version_only=True))
+    version_range = set(supported_versions(version_only=True)
+                        + old_versions(version_only=True))
     relop_seen = False
     for field in vstring.split(','):
         field = field.strip()
@@ -47,7 +52,13 @@ def parse_versions(vstring):
         ve = re.compile('(>=|<=|<<|=)? *(\d\.\d)$')
         m = ve.match(field)
         try:
+            if not m:
+                raise ValueError('error parsing Python-Version attribute')
             op, v = m.group(1), m.group(2)
+            vmaj, vmin = v.split('.')
+            # Don't silently ignore Python 3 versions.
+            if int(vmaj) > 2:
+                raise ValueError('error parsing Python-Version attribute, Python 3 version found')
             if op in (None, '='):
                 exact_versions.add(v)
             else:
@@ -56,10 +67,19 @@ def parse_versions(vstring):
                 version_range = [av for av in version_range if filtop(av ,v)]
         except Exception:
             raise ValueError, 'error parsing Python-Version attribute'
-    if 'versions' in vinfo:
-        vinfo['versions'] = exact_versions
-        if relop_seen:
-            vinfo['versions'] = exact_versions.union(version_range)
+    if add_exact:
+        if exact_versions:
+            vinfo['vexact'] = exact_versions
+        if 'versions' in vinfo:
+            if relop_seen:
+                vinfo['versions'] = set(version_range)
+            else:
+                del vinfo['versions']
+    else:
+        if 'versions' in vinfo:
+            vinfo['versions'] = exact_versions
+            if relop_seen:
+                vinfo['versions'] = exact_versions.union(version_range)
     return vinfo
 
 _old_versions = None
@@ -90,7 +110,9 @@ def unsupported_versions(version_only=False):
     else:
         return _unsupported_versions
 
-_supported_versions = None
+_supported_versions = ["python%s" % ver.strip() for ver in
+                       os.environ.get('DEBPYTHON_SUPPORTED', '').split(',')
+                       if ver.strip()]
 def supported_versions(version_only=False):
     global _supported_versions
     if not _supported_versions:
@@ -112,23 +134,48 @@ def supported_versions(version_only=False):
                 if line.startswith('Depends:'):
                     depends = line.split(':', 1)[1].strip().split(',')
             fd.close()
-            depends = [re.sub(r'\s*(\S+)[ (]?.*', r'\1', s) for s in depends]
-            _supported_versions = depends
+            if depends:
+                depends = [re.sub(r'\s*(\S+)[ (]?.*', r'\1', s) for s in depends]
+                _supported_versions = depends
+            if not _supported_versions:
+                # last resort: python-minimal not installed, apt-cache
+                # not available, hard code the value, #394084
+                _supported_versions = ['python2.6', 'python2.7']
     if version_only:
         return [v[6:] for v in _supported_versions]
     else:
         return _supported_versions
 
-_default_version = None
+_default_version = "python%s" % os.environ.get('DEBPYTHON_DEFAULT', '')
+if _default_version == 'python':
+    _default_version = None
 def default_version(version_only=False):
     global _default_version
     if not _default_version:
-        _default_version = link = os.readlink('/usr/bin/python')
-    # consistency check
-    debian_default = read_default('default-version')
-    if not _default_version in (debian_default, os.path.join('/usr/bin', debian_default)):
-        raise ValueError, "the symlink /usr/bin/python does not point to the python default version. It must be reset to point to %s" % debian_default
-    _default_version = debian_default
+        try:
+            _default_version = link = os.readlink('/usr/bin/python')
+        except OSError:
+            _default_version = None
+            try:
+                cmd = ['/usr/bin/python', '-c', 'import sys; print sys.version[:3]']
+                import subprocess
+                p = subprocess.Popen(cmd, bufsize=1,
+                                     shell=False, stdout=subprocess.PIPE)
+                fd = p.stdout
+            except ImportError:
+                fd = os.popen("/usr/bin/python -c 'import sys; print sys.version[:3]'")
+            line = fd.readline().strip()
+            fd.close()
+            if re.match(r'\d\.\d$', line):
+                _default_version = 'python' + line
+        # consistency check
+        try:
+            debian_default = read_default('default-version')
+        except ValueError:
+            debian_default = "python2.7"
+        if not _default_version in (debian_default, os.path.join('/usr/bin', debian_default)):
+            raise ValueError, "/usr/bin/python does not match the python default version. It must be reset to point to %s" % debian_default
+        _default_version = debian_default
     if version_only:
         return _default_version[6:]
     else:
@@ -136,28 +183,41 @@ def default_version(version_only=False):
 
 def requested_versions(vstring, version_only=False):
     versions = None
-    vinfo = parse_versions(vstring)
+    vinfo = parse_versions(vstring, add_exact=True)
     supported = supported_versions(version_only=True)
     if len(vinfo) == 1:
         if 'all' in vinfo:
             versions = supported
         elif 'current' in vinfo:
             versions = [default_version(version_only=True)]
+        elif 'vexact' in vinfo:
+            versions = vinfo['vexact']
         else:
             versions = vinfo['versions'].intersection(supported)
     elif 'all' in vinfo and 'current' in vinfo:
         raise ValueError, "both `current' and `all' in version string"
     elif 'all' in vinfo:
-        versions = versions = vinfo['versions'].intersection(supported)
+        if 'versions' in vinfo:
+            versions = vinfo['versions'].intersection(supported)
+        else:
+            versions = set(supported)
+        if 'vexact' in vinfo:
+            versions.update(vinfo['vexact'])
     elif 'current' in vinfo:
         current = default_version(version_only=True)
         if not current in vinfo['versions']:
             raise ValueError, "`current' version not in supported versions"
         versions = [current]
+    elif 'versions' in vinfo or 'vexact' in vinfo:
+        versions = set()
+        if 'versions' in vinfo:
+            versions = vinfo['versions'].intersection(supported)
+        if 'vexact' in vinfo:
+            versions.update(vinfo['vexact'])
     else:
-        raise ValueError, 'error in version string'
+        raise ValueError, 'No Python versions in version string'
     if not versions:
-        raise ValueError, 'empty set of versions'
+        raise ValueError('computed set of supported versions is empty')
     if version_only:
         return versions
     else:
@@ -181,15 +241,23 @@ class MissingVersionValueError(ValueError):
     pass
 
 def extract_pyversion_attribute(fn, pkg):
-    """read the debian/control file, extract the XS-Python-Version
-    field; check that XB-Python-Version exists for the package."""
+    """read the debian/control file, extract the X-Python-Version or
+    XS-Python-Version field; check that XB-Python-Version exists for the
+    package."""
 
     version = None
     sversion = None
     section = None
-    for line in file(fn):
+    try:
+        fp = file(fn, 'r')
+    except IOError, msg:
+        print "Cannot open %s: %s" % (fn, msg)
+        sys.exit(2)
+    for line in fp:
         line = line.strip()
         if line == '':
+            if section == None:
+                continue
             if pkg == 'Source':
                 break
             section = None
@@ -197,12 +265,12 @@ def extract_pyversion_attribute(fn, pkg):
             section = 'Source'
         elif line.startswith('Package: ' + pkg):
             section = pkg
-        elif line.startswith('XS-Python-Version:'):
+        elif line.lower().startswith(('xs-python-version:', 'x-python-version:')):
             if section != 'Source':
                 raise ValueError, \
-                      'attribute XS-Python-Version not in Source section'
+                      'attribute X(S)-Python-Version not in Source section'
             sversion = line.split(':', 1)[1].strip()
-        elif line.startswith('XB-Python-Version:'):
+        elif line.lower().startswith('xb-python-version:'):
             if section == pkg:
                 version = line.split(':', 1)[1].strip()
     if section == None:
@@ -210,11 +278,11 @@ def extract_pyversion_attribute(fn, pkg):
     if pkg == 'Source':
         if sversion == None:
             raise MissingVersionValueError, \
-                  'missing XS-Python-Version in control file'
+                  'no X(S)-Python-Version in control file'
         return sversion
     if version == None:
         raise MissingVersionValueError, \
-              'missing XB-Python-Version for package `%s' % pkg
+              'no XB-Python-Version for package `%s' % pkg
     return version
 
 # compatibility functions to parse debian/pyversions
@@ -266,7 +334,7 @@ def main():
                       help='print the supported python versions',
                       action='store_true', dest='supported')
     parser.add_option('-r', '--requested',
-                      help='print the python versions requested by a build; the argument is either the name of a control file or the value of the XS-Python-Version attribute',
+                      help='print the python versions requested by a build; the argument is either the name of a control file or the value of the X(S)-Python-Version attribute',
                       action='store_true', dest='requested')
     parser.add_option('-i', '--installed',
                       help='print the installed supported python versions',
@@ -304,7 +372,7 @@ def main():
                     sys.exit(1)
                 except MissingVersionValueError:
                     fn = os.path.join(os.path.dirname(fn), 'pyversions')
-                    sys.stderr.write("%s: missing XS-Python-Version in control file, fall back to %s\n" \
+                    sys.stderr.write("%s: missing X(S)-Python-Version in control file, fall back to %s\n" \
                                      % (program, fn))
                     try:
                         vstring = extract_pyversion_attribute_bis(fn)
@@ -313,6 +381,9 @@ def main():
                         sys.stderr.write("%s: missing debian/pyversions file, fall back to supported versions\n" \
                                          % program)
                         vs = supported_versions(opts.version_only)
+                except ValueError, e:
+                    sys.stderr.write("%s: %s\n" % (program, e))
+                    sys.exit(4)
             else:
                 vs = requested_versions(versions, opts.version_only)
             print ' '.join(vs)
